@@ -75,8 +75,8 @@ public:
 /*     // --- */
 
     int m_iCharacters;
-    std::vector<SGroupInfo *> m_vGroups;
     SGroupInfo *m_pBaseGroup;
+    int iNumChildNodes;
 
     std::vector<character> m_vCharacters;
 
@@ -90,7 +90,7 @@ public:
     std::string m_strDefaultContext;
   };
 
-  CAlphIO(std::string SystemLocation, std::string UserLocation, std::vector < std::string > Filenames);
+  CAlphIO(std::string SystemLocation, std::string UserLocation, std::vector < std::string > &Filenames);
   ~CAlphIO();
   void GetAlphabets(std::vector < std::string > *AlphabetList) const;
   std::string GetDefault();
@@ -98,6 +98,7 @@ public:
   void SetInfo(const AlphInfo & NewInfo);
   void Delete(const std::string & AlphID);
 private:
+  std::vector<SGroupInfo *> m_vGroups;
   AlphInfo BlankInfo;
   std::string SystemLocation;
   std::string UserLocation;
