@@ -1,42 +1,45 @@
 // Alphabet.h
 //
-/////////////////////////////////////////////////////////////////////////////
-// Alphabet.h
+// Copyright (c) 2007 The Dasher Team
 //
-/////////////////////////////////////////////////////////////////////////////
+// This file is part of Dasher.
 //
-// Copyright (c) 2001-2002 David Ward
+// Dasher is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
 //
-/////////////////////////////////////////////////////////////////////////////
+// Dasher is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Dasher; if not, write to the Free Software 
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #ifndef __DASHER_ALPHABET_H__
 #define __DASHER_ALPHABET_H__
 
+#include "AlphIO.h"
+#include "AlphabetMap.h"
 #include "../DasherTypes.h"
 #include "GroupInfo.h"
 
 #include <cstdlib>
+#include <iostream>
 #include <vector>
-#include "AlphabetMap.h"
-#include "AlphIO.h"
 
 namespace Dasher {
-
   ///
   /// \defgroup Alphabet Alphabet information
   /// @{
 
   class CAlphabet {
   public:
-
-
     CAlphabet();
-
     CAlphabet(const CAlphIO::AlphInfo & AlphInfo);
 
-     ~CAlphabet() {
-    };
-
     // Return size of alphabet, including control symbols
     int GetNumberSymbols() const {
       return m_Characters.size();
@@ -59,7 +62,7 @@ namespace Dasher {
       return m_DefaultEncoding;
     }
 
-    std::string & GetTrainingFile() {
+    const std::string & GetTrainingFile() const {
       return m_TrainingFile;
     }
     std::string GetGameModeFile() {
@@ -74,7 +77,6 @@ namespace Dasher {
     symbol GetControlSymbol() const;
     symbol GetStartConversionSymbol() const;
     symbol GetEndConversionSymbol() const;
-    symbol GetResetSymbol() const;
 
     const std::string & GetDisplayText(symbol i) const {
       return m_Display[i];
@@ -103,36 +105,28 @@ namespace Dasher {
 
     //int get_group(symbol i) const {return m_Group[i];}                
     // return group membership of i'th symbol
+    
+    class SymbolStream {
+    public:
+      SymbolStream(const CAlphabet *pAlph, std::istream &_in);
+      symbol next();
+    private:
+      void readMore();
+      const alphabet_map &map;
+      char buf[1024];
+      int pos, len;
+      std::istream &in;
+    };
+    
     // Fills Symbols with the symbols corresponding to Input. {{{ Note that this
     // is not necessarily reversible by repeated use of GetText. Some text
-    // may not be recognised and so discarded. If IsMore is true then Input
-    // is truncated to any final characters that were not used due to ambiguous
-    // continuation. If IsMore is false Input is assumed to be all the available
-    // text and so a symbol will be returned for a final "a" even if "ae" is
-    // defined as its own symbol. }}}
-    void GetSymbols(std::vector<symbol> *Symbols, std::string * Input, bool IsMore) const;
-
-
-    /// Look up symbols corresponding to string. Cannot cope with
-    /// parial input, but knows about UTF-8, so can include 'unknown
-    /// symbol' as 0 in the output. Slower than GetSymbols, so don't
-    /// use for import, but is useful for looking up contexts.
-    void GetSymbolsFull(std::vector<symbol > *Symbols, std::string *Input) const;
+    // may not be recognised and so discarded. }}}
 
+    void GetSymbols(std::vector<symbol> &Symbols, const std::string &Input) const;
+    //SymbolStream *GetSymbols(std::istream &in) const;
 
     void Trace() const;         // diagnostic
 
-    // Add the characters that can appear in Nodes
-    void AddChar(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);    // add single char to the alphabet
-
-    // Alphabet language parameters
-    void AddParagraphSymbol(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);
-    void AddSpaceSymbol(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);
-    void AddControlSymbol(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);
-    void AddStartConversionSymbol(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);
-    void AddEndConversionSymbol(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);
-    void AddResetSymbol(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);
-
     void SetOrientation(Opts::ScreenOrientations Orientation) {
       m_Orientation = Orientation;
     }
@@ -149,52 +143,57 @@ namespace Dasher {
       m_DefaultPalette = Palette;
     }
 
-    const alphabet_map GetAlphabetMap() const {
-      return TextMap;
-    } 
-
     const std::string &GetDefaultContext() const {
       return m_strDefaultContext;
     }
 
     SGroupInfo *m_pBaseGroup;
-    
-    
+    int iNumChildNodes;
   private:
 
+    // Add the characters that can appear in Nodes
+    void AddChar(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);    // add single char to the alphabet
+
+    // Alphabet language parameters
+    void AddParagraphSymbol(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);
+    void AddSpaceSymbol(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);
+    void AddControlSymbol(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);
+    void AddStartConversionSymbol(std::string NewCharacter, std::string Display , int Colour, std::string Foreground);
+    void AddEndConversionSymbol(std::string NewCharacter, std::string Display, int Colour, std::string Foreground);
+
     Opts::AlphabetTypes m_DefaultEncoding;
     Opts::ScreenOrientations m_Orientation;
     symbol m_ParagraphSymbol;
     symbol m_SpaceSymbol;
     symbol m_ControlSymbol;
-    //-- Added for Kanji Conversion 13 July 2005 by T.Kaburagi
     symbol m_StartConversionSymbol;
     symbol m_EndConversionSymbol;
-    symbol m_iResetSymbol;
 
     std::string m_TrainingFile;
     std::string m_GameModeFile;
     std::string m_DefaultPalette;
 
+    class utf8_length
+      {
+        public:
+          utf8_length();
+          int operator[](const unsigned char) const;
+          int max_length;
+        private:
+          int utf8_count_array[0x100];
+      };
+    static utf8_length m_utf8_count_array;
+
+    // TODO: This is inane
     std::vector < std::string > m_Characters;   // stores the characters
     std::vector < std::string > m_Display;      // stores how the characters are visually represented in the Dasher nodes
     std::vector < int >m_Colours;       // stores the colour of the characters
     std::vector < std::string > m_Foreground;   // stores the colour of the character foreground
-
-    //    int m_iGroups;              // number of groups
-
-/*     std::vector < int >m_GroupStart;    // stores the group start index */
-/*     std::vector < int >m_GroupEnd;      // stores the group end index (1 past the last) */
-/*     std::vector < int >m_GroupColour;   // stores the colour of the group */
-/*     std::vector < std::string > m_GroupLabel; */
-
-    SGroupInfo *pFirstGroup;
+    // ----
 
     alphabet_map TextMap;
 
     std::string m_strDefaultContext;
-
-    //    friend class CGroupAdder;
   };
 
   /// @}
@@ -219,10 +218,6 @@ namespace Dasher {
     return m_EndConversionSymbol;
   }
   
-  inline symbol CAlphabet::GetResetSymbol() const {
-    return m_iResetSymbol;
-  }
-
 
 }                              // end namespace dasher
 
