@@ -225,10 +225,6 @@ public:
 
   void PauseAt(int MouseX, int MouseY); // are required to make
 
-  /// Halt Dasher. This simply freezes Dasher but does not emit a stop event, so does not result in speech etc.
-
-  void Halt();
-
   /// Unpause Dasher
   /// \param Time Time in ms, used to keep a constant frame rate
 
@@ -300,8 +296,6 @@ public:
 
   double GetFramerate();
 
-  int GetRenderCount();
-
   /// @}
 
   /// @name Control hierarchy and action buttons
@@ -361,8 +355,6 @@ public:
     m_bRedrawScheduled = true; 
   }; 
 
-  void CheckRedraw();
-
   std::string GetContext(int iStart, int iLength);
 
   void SetControlOffset(int iOffset);
@@ -393,6 +385,12 @@ protected:
   ///
 
   void OnUIRealised();
+	
+  ///
+  /// Creates a default set of modules. Override in subclasses to create any
+  /// extra/different modules specific to the platform (eg input device drivers)
+  ///	
+  virtual void CreateModules();
 
   /// @}
 
@@ -439,6 +437,9 @@ protected:
 
  private:
 
+  //The default expansion policy to use - an amortized policy depending on the LP_NODE_BUDGET parameter. 
+  CExpansionPolicy *m_defaultPolicy;
+  
   /// @name Platform dependent utility functions 
   /// These functions provide various platform dependent functions
   /// required by the core. A derived class is created for each
@@ -474,13 +475,6 @@ protected:
 
   virtual void SetupUI() = 0;
 
-  ///
-  /// Create any module factories which are specific to the platform
-  /// (eg input device drivers)
-  ///
-
-  virtual void CreateLocalFactories() = 0;
-
   ///
   /// Create settings store object, which will be platform dependent
   /// TODO: Can this not be done just by selecting which settings
@@ -524,8 +518,7 @@ protected:
   /// Provide a new CDasherInput input device object.
 
   void CreateInput();
-
-  void CreateFactories();
+	
   void CreateInputFilter();
 
   void CreateModel(int iOffset);
@@ -534,44 +527,14 @@ protected:
   void ChangeAlphabet();
   void ChangeColours();
   void ChangeView();
-  void Redraw(bool bRedrawNodes);
+  void Redraw(bool bRedrawNodes, CExpansionPolicy &policy);
   void SetupActionButtons();
   void DestroyActionButtons();
   void PositionActionButtons();
   bool DrawActionButtons();
 
-  void LeaveState(EState iState);
-  void EnterState(EState iState);
-
   void WriteTrainFileFull();
   void WriteTrainFilePartial();
-
-  
-  /// @name Lock Management
-  /// Functions for locking/unlocking the core. Note that the lock
-  /// flags parameter is currently ignored, but will be used to give
-  /// finer granularity
-  /// @{
-
-  ///
-  /// Add a lock
-  ///
-
-  int AddLock(const std::string &strDisplay);
-
-  ///
-  /// Release an existing lock. Note that these functions have minimal
-  /// error checking at the moment, so be careful.
-  ///
-
-  void ReleaseLock(int iLockID);
-
-  ///
-  /// Return the lock to display, or NULL for no locks present
-  ///
-
-  SLockData *GetCurrentLock(); 
-  /// @}
   
   std::deque<std::string> m_deGameModeStrings;
 
@@ -593,7 +556,6 @@ protected:
   CColourIO *m_ColourIO;
   CNodeCreationManager *m_pNCManager;
   CUserLogBase *m_pUserLog; 
-  //  CTrainingHelper *m_pTrainingHelper;
   /// @}
 
   std::string strTrainfileBuffer;
@@ -606,7 +568,6 @@ protected:
   /// @{
   //  bool m_bGlobalLock; // The big lock
   bool m_bRedrawScheduled;
-  int m_iLockCount;
   EState m_iCurrentState;
   bool m_bOldVisible;
 
