@@ -29,10 +29,6 @@
 
 #include "Event.h"
 #include "DasherInterfaceBase.h"
-#include "LanguageModelling/PPMLanguageModel.h"
-#include "LanguageModelling/WordLanguageModel.h"
-#include "LanguageModelling/DictLanguageModel.h"
-#include "LanguageModelling/MixtureLanguageModel.h"
 #include "NodeCreationManager.h"
 #include "DasherGameMode.h"
 #include "AlphabetManager.h"
@@ -59,7 +55,7 @@ CDasherModel::CDasherModel(CEventHandler *pEventHandler,
 			   CNodeCreationManager *pNCManager,
 			   CDasherInterfaceBase *pDasherInterface,
 			   CDasherView *pView, int iOffset)
-  : CDasherComponent(pEventHandler, pSettingsStore) {
+  : CFrameRate(pEventHandler, pSettingsStore) {
   m_pNodeCreationManager = pNCManager;
   m_pDasherInterface = pDasherInterface;
 
@@ -73,7 +69,7 @@ CDasherModel::CDasherModel(CEventHandler *pEventHandler,
 
   m_Rootmin = 0;
   m_Rootmax = 0;
-  m_iTargetOffset = 0;
+  m_iDisplayOffset = 0;
   m_dTotalNats = 0.0;
  
   // TODO: Need to rationalise the require conversion methods
@@ -85,9 +81,6 @@ CDasherModel::CDasherModel(CEventHandler *pEventHandler,
 
   SetBoolParameter(BP_CONVERSION_MODE, m_bRequireConversion);
 
-  // Set max bitrate in the FrameRate class
-  m_dMaxRate = GetLongParameter(LP_MAX_BITRATE) / 100.0;
-  m_fr.SetMaxBitrate(m_dMaxRate);
   m_dAddProb = 0.003;
 
   int iNormalization = GetLongParameter(LP_NORMALIZATION);
@@ -112,28 +105,25 @@ CDasherModel::~CDasherModel() {
 }
 
 void CDasherModel::HandleEvent(Dasher::CEvent *pEvent) {
+  CFrameRate::HandleEvent(pEvent);
 
   if(pEvent->m_iEventType == EV_PARAM_NOTIFY) {
     Dasher::CParameterNotificationEvent * pEvt(static_cast < Dasher::CParameterNotificationEvent * >(pEvent));
 
     switch (pEvt->m_iParameter) {
-    case LP_MAX_BITRATE: // Delibarate fallthrough
-    case LP_BOOSTFACTOR: // Deliberate fallthrough
-    case LP_SPEED_DIVISOR:
-      m_dMaxRate = GetLongParameter(LP_MAX_BITRATE) * GetLongParameter(LP_BOOSTFACTOR) / 100 / static_cast<double>(GetLongParameter(LP_SPEED_DIVISOR));
-      m_fr.SetMaxBitrate(m_dMaxRate);
-      break;
     case BP_CONTROL_MODE: // Rebuild the model if control mode is switched on/off
-      RebuildAroundNode(Get_node_under_crosshair());
+      RebuildAroundCrosshair();
       break;
-    case BP_DELAY_VIEW:
-      MatchTarget(GetBoolParameter(BP_DELAY_VIEW));
+    case BP_SMOOTH_OFFSET:
+      if (!GetBoolParameter(BP_SMOOTH_OFFSET))
+        //smoothing has just been turned off. End any transition/jump currently
+        // in progress at it's current point
+        AbortOffset();
       break;
     case BP_DASHER_PAUSED:
       if(GetBoolParameter(BP_SLOW_START))
-	m_iStartTime = 0;
-      else
-	m_iStartTime = 1;
+	TriggerSlowdown();
+      //else, leave m_iStartTime as is - will result in no slow start
       break;
     case BP_GAME_MODE:
       m_bGameMode = GetBoolParameter(BP_GAME_MODE);
@@ -157,13 +147,10 @@ void CDasherModel::HandleEvent(Dasher::CEvent *pEvent) {
 }
 
 void CDasherModel::Make_root(CDasherNode *pNewRoot) {
-  // TODO: Note that subnodes can be the root transiently during the
-  // re-rooting process.
-
   //  std::cout << "Make root" << std::endl;
 
   DASHER_ASSERT(pNewRoot != NULL);
-  DASHER_ASSERT(pNewRoot->NodeIsParent(m_Root));
+  DASHER_ASSERT(pNewRoot->Parent() == m_Root);
 
   m_Root->SetFlag(NF_COMMITTED, true);
 
@@ -172,8 +159,7 @@ void CDasherModel::Make_root(CDasherNode *pNewRoot) {
   oldroots.push_back(m_Root);
 
   // TODO: tidy up conditional
-  while(((oldroots.size() > 10) && (!m_bRequireConversion || (oldroots[0]->GetFlag(NF_CONVERTED)))) || 
-	(oldroots[0]->GetFlag(NF_SUBNODE))) {
+  while((oldroots.size() > 10) && (!m_bRequireConversion || (oldroots[0]->GetFlag(NF_CONVERTED)))) {
     oldroots[0]->OrphanChild(oldroots[1]);
     delete oldroots[0];
     oldroots.pop_front();
@@ -203,28 +189,24 @@ void CDasherModel::RecursiveMakeRoot(CDasherNode *pNewRoot) {
   // TODO: we really ought to check that pNewRoot is actually a
   // descendent of the root, although that should be guaranteed
 
-  if(!pNewRoot->NodeIsParent(m_Root))
+  if(pNewRoot->Parent() != m_Root)
     RecursiveMakeRoot(pNewRoot->Parent());
 
   Make_root(pNewRoot);
 }
 
-// RebuildAroundNode is only used when BP_CONTROL changes,
-// so not very often.
-void CDasherModel::RebuildAroundNode(CDasherNode *pNode) {
+// only used when BP_CONTROL changes, so not very often.
+void CDasherModel::RebuildAroundCrosshair() {
+  CDasherNode *pNode = Get_node_under_crosshair();
   DASHER_ASSERT(pNode != NULL);
-
-  while(pNode->GetFlag(NF_SUBNODE))
-    pNode = pNode->Parent();
+  DASHER_ASSERT(pNode == m_pLastOutput);
 
   RecursiveMakeRoot(pNode);
-
+  DASHER_ASSERT(m_Root == pNode);
   ClearRootQueue();
   m_Root->Delete_children();
 
-  m_Root->m_pNodeManager->PopulateChildren(m_Root);
-
-  m_pLastOutput = m_Root;
+  m_Root->PopulateChildren();
 }
 
 void CDasherModel::Reparent_root(int lower, int upper) {
@@ -236,24 +218,17 @@ void CDasherModel::Reparent_root(int lower, int upper) {
   CDasherNode *pNewRoot;
 
   if(oldroots.size() == 0) {
-    pNewRoot = m_Root->m_pNodeManager->RebuildParent(m_Root);
+    pNewRoot = m_Root->RebuildParent();
   }
   else {
     pNewRoot = oldroots.back();
     oldroots.pop_back();
-
-    while((oldroots.size() > 0) && pNewRoot->GetFlag(NF_SUBNODE)) {
-      pNewRoot = oldroots.back();
-      oldroots.pop_back();
-    }
   }
 
   // Return if there's no existing parent and no way of recreating one
   if(pNewRoot == NULL)
     return;
 
-  DASHER_ASSERT(!(pNewRoot->GetFlag(NF_SUBNODE)));
-
   pNewRoot->SetFlag(NF_COMMITTED, false);
 
   CDasherNode *pCurrent = m_Root;
@@ -270,11 +245,10 @@ void CDasherModel::Reparent_root(int lower, int upper) {
     myint iRootWidth = m_Rootmax - m_Rootmin;
     
     // Fail and undo root creation if the new root is bigger than allowed by normalisation
-    if(!(pNewRoot->GetFlag(NF_SUBNODE)) && 
-       (((myint((GetLongParameter(LP_NORMALIZATION) - upper)) / static_cast<double>(iWidth)) >
+    if(((myint((GetLongParameter(LP_NORMALIZATION) - upper)) / static_cast<double>(iWidth)) >
 	 (m_Rootmax_max - m_Rootmax)/static_cast<double>(iRootWidth)) || 
 	((myint(lower) / static_cast<double>(iWidth)) > 
-	 (m_Rootmin - m_Rootmin_min) / static_cast<double>(iRootWidth)))) {
+	 (m_Rootmin - m_Rootmin_min) / static_cast<double>(iRootWidth))) {
       pNewRoot->OrphanChild(m_Root);
       delete pNewRoot;
       return;
@@ -310,13 +284,13 @@ void CDasherModel::ClearRootQueue() {
 CDasherNode *CDasherModel::Get_node_under_crosshair() {
   DASHER_ASSERT(m_Root != NULL);
 
-  return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
+  return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iDisplayOffset, m_Rootmax + m_iDisplayOffset, GetLongParameter(LP_OX), GetLongParameter(LP_OY));
 }
 
 CDasherNode *CDasherModel::Get_node_under_mouse(myint Mousex, myint Mousey) {
   DASHER_ASSERT(m_Root != NULL);
 
-  return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, Mousex, Mousey);
+  return m_Root->Get_node_under(GetLongParameter(LP_NORMALIZATION), m_Rootmin + m_iDisplayOffset, m_Rootmax + m_iDisplayOffset, Mousex, Mousey);
 }
 
 void CDasherModel::DeleteTree() {
@@ -328,31 +302,12 @@ void CDasherModel::DeleteTree() {
 void CDasherModel::InitialiseAtOffset(int iOffset, CDasherView *pView) {
   DeleteTree();
 
-  if(iOffset == 0)
-    m_Root = m_pNodeCreationManager->GetRoot(0, NULL, 0,GetLongParameter(LP_NORMALIZATION), NULL);
-  else {
-    // Get the most recent character
-    std::string strContext = m_pDasherInterface->GetContext(iOffset - 1, 1);
-
-    CAlphabetManager::SRootData oData;
-
-    oData.szContext = new char[strContext.size() + 1];
-    strcpy(oData.szContext, strContext.c_str());
-
-    oData.iOffset = iOffset;
-
-    m_Root = m_pNodeCreationManager->GetRoot(0, NULL, 0, GetLongParameter(LP_NORMALIZATION), &oData);
-
-    delete[] oData.szContext;
-  }
-
-  m_pLastOutput = m_Root;
-
-  // Create children of the root
-  // TODO: What about parents?
-
-  Recursive_Push_Node(m_Root, 0);
-
+  m_Root = m_pNodeCreationManager->GetAlphRoot(NULL, 0,GetLongParameter(LP_NORMALIZATION), iOffset!=0, iOffset);
+  m_pLastOutput = (m_Root->GetFlag(NF_SEEN)) ? m_Root : NULL;
+  
+  // Create children of the root...
+  ExpandNode(m_Root);
+	
   // Set the root coordinates so that the root node is an appropriate
   // size and we're not in any of the children
 
@@ -363,8 +318,9 @@ void CDasherModel::InitialiseAtOffset(int iOffset, CDasherView *pView) {
   m_Rootmin = GetLongParameter(LP_MAX_Y) / 2 - iWidth / 2;
   m_Rootmax = GetLongParameter(LP_MAX_Y) / 2 + iWidth / 2;
 
-  m_iTargetOffset = 0;
+  m_iDisplayOffset = 0;
 
+  //now (re)create parents, while they show on the screen
   if(pView) {
     while(pView->IsNodeVisible(m_Rootmin,m_Rootmax)) {
       CDasherNode *pOldRoot = m_Root;
@@ -383,11 +339,12 @@ void CDasherModel::Get_new_root_coords(dasherint X, dasherint Y, dasherint &r1,
   if(m_iStartTime == 0)
     m_iStartTime = iTime;
 
-  int iSteps = m_fr.Steps();
+  int iSteps = Steps();
 
   double dFactor;
 
-  if(IsSlowdown(iTime))
+  if(GetBoolParameter(BP_SLOW_START) &&
+     ((iTime - m_iStartTime) < GetLongParameter(LP_SLOW_START_TIME)))
     dFactor = 0.1 * (1 + 9 * ((iTime - m_iStartTime) / static_cast<double>(GetLongParameter(LP_SLOW_START_TIME))));
   else 
     dFactor = 1.0;
@@ -405,7 +362,7 @@ void CDasherModel::Get_new_root_coords(dasherint X, dasherint Y, dasherint &r1,
   // new root{min,max} r1,r2, old root{min,max} R1,R2
   const dasherint R1 = m_Rootmin;
   const dasherint R2 = m_Rootmax;
-  const dasherint Y1 = 0;
+  // const dasherint Y1 = 0;
   dasherint Y2(GetLongParameter(LP_MAX_Y));
   dasherint iOX(GetLongParameter(LP_OX));
   dasherint iOY(GetLongParameter(LP_OY));
@@ -439,7 +396,7 @@ void CDasherModel::Get_new_root_coords(dasherint X, dasherint Y, dasherint &r1,
   // Calculate the minimum size of the viewport corresponding to the
   // maximum zoom.
 
-  dasherint iMinSize(m_fr.MinSize(Y2, dFactor));
+  dasherint iMinSize(MinSize(Y2, dFactor));
 
   if((y2 - y1) < iMinSize) {
     newy1 = y1 * (Y2 - iMinSize) / (Y2 - (y2 - y1));
@@ -465,36 +422,32 @@ void CDasherModel::Get_new_root_coords(dasherint X, dasherint Y, dasherint &r1,
   r2 = ((R2 - C) * Y2) / (y2 - y1) + C;
 }
 
-bool CDasherModel::UpdatePosition(myint miMousex, myint miMousey, unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
-  // Clear out parameters that might get passed in to track user activity
-  if (pAdded != NULL)
-    pAdded->clear();
-  if (pNumDeleted != NULL)
-    *pNumDeleted = 0;
+bool CDasherModel::NextScheduledStep(unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted)
+{
+  DASHER_ASSERT(GetBoolParameter(BP_DASHER_PAUSED));
+  if (m_deGotoQueue.size() == 0) return false;
+  myint iNewMin, iNewMax;
+  iNewMin = m_deGotoQueue.front().iN1;
+  iNewMax = m_deGotoQueue.front().iN2;
+  m_deGotoQueue.pop_front();
 
-  if(GetBoolParameter(BP_DASHER_PAUSED) && (m_deGotoQueue.size() == 0))
-    return false;
+  UpdateBounds(iNewMin, iNewMax, iTime, pAdded, pNumDeleted);
+  return true;
+}
 
-  // Find out the current node under the crosshair
-  CDasherNode *old_under_cross=Get_node_under_crosshair();
+void CDasherModel::OneStepTowards(myint miMousex, myint miMousey, unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
+  //if (GetBoolParameter(BP_DASHER_PAUSED)) return false;
+  m_deGotoQueue.clear();
 
-
-  myint iNewMin;
-  myint iNewMax;
-
-  if(m_deGotoQueue.size() == 0) {
-    // works out next viewpoint
-    Get_new_root_coords(miMousex, miMousey, iNewMin, iNewMax, iTime);
+  myint iNewMin, iNewMax;
+  // works out next viewpoint
+  Get_new_root_coords(miMousex, miMousey, iNewMin, iNewMax, iTime);
   
-    if(GetBoolParameter(BP_OLD_STYLE_PUSH))
-      OldPush(miMousex, miMousey);
-  }
-  else {
-    iNewMin = m_deGotoQueue.front().iN1;
-    iNewMax = m_deGotoQueue.front().iN2;
-    m_deGotoQueue.pop_front();
-  }
+  UpdateBounds(iNewMin, iNewMax, iTime, pAdded, pNumDeleted);
+}
 
+void CDasherModel::UpdateBounds(myint iNewMin, myint iNewMax, unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
+  
   m_dTotalNats += log((iNewMax - iNewMin) / static_cast<double>(m_Rootmax - m_Rootmin));
 
   // Now actually zoom to the new location
@@ -502,99 +455,41 @@ bool CDasherModel::UpdatePosition(myint miMousex, myint miMousey, unsigned long
 
 
   // Check whether new nodes need to be created
-  CDasherNode* new_under_cross = Get_node_under_crosshair();
-  Push_Node(new_under_cross);
+  ExpandNode(Get_node_under_crosshair());
   
-  // TODO: Make this more efficient
-  new_under_cross = Get_node_under_crosshair();
-
-  HandleOutput(new_under_cross, old_under_cross, pAdded, pNumDeleted);
-
-
-  return true;
+// This'll get done again when we render the frame, later, but we use NF_SEEN
+// (set here) to ensure the node under the cursor can never be collapsed
+// (even when the node budget is exceeded) when we do the rendering...
+  HandleOutput(pAdded, pNumDeleted);
 }
 
 void CDasherModel::NewFrame(unsigned long Time) {
-  m_fr.NewFrame(Time);
+  CFrameRate::NewFrame(Time);
   ///GAME MODE TEMP///Pass new frame events onto our teacher
   GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher();
   if(m_bGameMode && pTeacher)
     pTeacher->NewFrame(Time);
 }
 
-void CDasherModel::OldPush(myint iMousex, myint iMousey) {
-  // push node under mouse
-  CDasherNode *pUnderMouse = Get_node_under_mouse(iMousex, iMousey);
-
-  Push_Node(pUnderMouse);
-
-  if(Framerate() > 4) {
-    // push node under mouse but with x coord on RHS
-    CDasherNode *pRight = Get_node_under_mouse(50, iMousey);
-    Push_Node(pRight);
-  }
-
-  if(Framerate() > 8) {
-    // push node under the crosshair
-    CDasherNode *pUnderCross = Get_node_under_crosshair();
-    Push_Node(pUnderCross);
-  }
-
-  int iRandom = RandomInt();
-
-  if(Framerate() > 8) {
-    // add some noise and push another node
-    CDasherNode *pRight = Get_node_under_mouse(50, iMousey + iRandom % 500 - 250);
-    Push_Node(pRight);
-  }
-
-  iRandom = RandomInt();
-
-  if(Framerate() > 15) {
-    // add some noise and push another node
-    CDasherNode *pRight = Get_node_under_mouse(50, iMousey + iRandom % 500 - 250);
-    Push_Node(pRight);
-  }
-
-  // only do this is Dasher is flying
-  if(Framerate() > 30) {
-    for(int i = 1; i < int (Framerate() - 30) / 3; i++) {
-
-      int iRandom = RandomInt();
-      
-      if(Framerate() > 8) {
-	// add some noise and push another node
-      	CDasherNode *pRight = Get_node_under_mouse(50, iMousey + iRandom % 500 - 250);
-	Push_Node(pRight);
-      }
-      
-      iRandom = RandomInt();
-      // push at a random node on the RHS
-      CDasherNode *pRight = Get_node_under_mouse(50, iMousey + iRandom % 1000 - 500);
-      Push_Node(pRight);
-
-    }
-  }
-}
-
 void CDasherModel::RecursiveOutput(CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* pAdded) {
-  if(pNode->Parent() && (!pNode->Parent()->GetFlag(NF_SEEN)))
-    RecursiveOutput(pNode->Parent(), pAdded);
-
-  if(pNode->Parent())
-    pNode->Parent()->m_pNodeManager->Leave(pNode->Parent());
-
-  pNode->m_pNodeManager->Enter(pNode);
+  if(pNode->Parent()) {
+    if (!pNode->Parent()->GetFlag(NF_SEEN))
+      RecursiveOutput(pNode->Parent(), pAdded);
 
+    pNode->Parent()->Leave();
+  }
+  
+  pNode->Enter();
+  
+  m_pLastOutput = pNode;
   pNode->SetFlag(NF_SEEN, true);
-  pNode->m_pNodeManager->Output(pNode, pAdded, GetLongParameter(LP_NORMALIZATION));
+  pNode->Output(pAdded, GetLongParameter(LP_NORMALIZATION));
 
   // If the node we are outputting is the last one in a game target sentence, then
   // notify the game mode teacher.
   if(m_bGameMode)
     if(pNode->GetFlag(NF_END_GAME))
       GameMode::CDasherGameMode::GetTeacher()->SentenceFinished();
-
 }
 
 void CDasherModel::NewGoTo(myint newRootmin, myint newRootmax, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
@@ -602,11 +497,11 @@ void CDasherModel::NewGoTo(myint newRootmin, myint newRootmax, Dasher::VECTOR_SY
   // Update the max and min of the root node to make iTargetMin and
   // iTargetMax the edges of the viewport.
 
-  if(newRootmin + m_iTargetOffset > (myint)GetLongParameter(LP_MAX_Y) / 2 - 1)
-    newRootmin = (myint)GetLongParameter(LP_MAX_Y) / 2 - 1 - m_iTargetOffset;
+  if(newRootmin + m_iDisplayOffset > (myint)GetLongParameter(LP_MAX_Y) / 2 - 1)
+    newRootmin = (myint)GetLongParameter(LP_MAX_Y) / 2 - 1 - m_iDisplayOffset;
 
-  if(newRootmax + m_iTargetOffset < (myint)GetLongParameter(LP_MAX_Y) / 2 + 1)
-    newRootmax = (myint)GetLongParameter(LP_MAX_Y) / 2 + 1 - m_iTargetOffset;
+  if(newRootmax + m_iDisplayOffset < (myint)GetLongParameter(LP_MAX_Y) / 2 + 1)
+    newRootmax = (myint)GetLongParameter(LP_MAX_Y) / 2 + 1 - m_iDisplayOffset;
 
   // Check that we haven't drifted too far. The rule is that we're not
   // allowed to let the root max and min cross the midpoint of the
@@ -624,7 +519,7 @@ void CDasherModel::NewGoTo(myint newRootmin, myint newRootmax, Dasher::VECTOR_SY
     m_Rootmax = newRootmax;
     m_Rootmin = newRootmin;
 
-    m_iTargetOffset = (m_iTargetOffset * 90) / 100;
+    m_iDisplayOffset = (m_iDisplayOffset * 90) / 100;
   }
   else {
     // TODO - force a new root to be chosen, so that we get better
@@ -632,127 +527,55 @@ void CDasherModel::NewGoTo(myint newRootmin, myint newRootmax, Dasher::VECTOR_SY
   }
 }
 
-void CDasherModel::HandleOutput(CDasherNode *pNewNode, CDasherNode *pOldNode, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
+void CDasherModel::HandleOutput(Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted) {
+  CDasherNode *pNewNode = Get_node_under_crosshair();
   DASHER_ASSERT(pNewNode != NULL);
-  //  DASHER_ASSERT(pOldNode != NULL);
   
-  //if(pNewNode != m_pLastOutput)
-  //  std::cout << "HandleOutput: " << pOldNode << " => " << pNewNode << std::endl;
+  //  std::cout << "HandleOutput: " << m_pLastOutput << " => " << pNewNode << std::endl;
   
-  if(pNewNode != m_pLastOutput)
-    DeleteCharacters(pNewNode, m_pLastOutput, pNumDeleted);
+  CDasherNode *pLastSeen = pNewNode;
+  while (pLastSeen && !pLastSeen->GetFlag(NF_SEEN))
+    pLastSeen = pLastSeen->Parent();
   
-  if(pNewNode->GetFlag(NF_SEEN)) {
-    m_pLastOutput = pNewNode;
-    return;
-  }
-
-  RecursiveOutput(pNewNode, pAdded);
-
-  m_pLastOutput = pNewNode;
-}
-
-bool CDasherModel::DeleteCharacters(CDasherNode *newnode, CDasherNode *oldnode, int* pNumDeleted) {
-  DASHER_ASSERT(newnode != NULL);
-  DASHER_ASSERT(oldnode != NULL);
-
-  //  std::cout << oldnode << " " << newnode << std::endl;
-
-  // This deals with the trivial instance - we're reversing back over
-  // text that we've seen already
-  if(newnode->GetFlag(NF_SEEN)) {
-    // TODO: I really cannot believe that  the following could ever have worked.
-    
-    //   if(oldnode->Parent() == newnode) {
-    //       std::cout << "DCA" << std::endl;
-    //       oldnode->m_pNodeManager->Undo(oldnode);
-    //       oldnode->Parent()->m_pNodeManager->Enter(oldnode->Parent());
-    //       if (pNumDeleted != NULL)
-    //         (*pNumDeleted)++;
-    //       oldnode->SetFlag(NF_SEEN, false);
-    //       return true;
-    //     }
-    //     if(DeleteCharacters(newnode, oldnode->Parent(), pNumDeleted) == true) {
-    //       std::cout << "DCB" << std::endl;
-    //       oldnode->m_pNodeManager->Undo(oldnode);
-    //       oldnode->Parent()->m_pNodeManager->Enter(oldnode->Parent());
-    //       if (pNumDeleted != NULL)
-    // 	(*pNumDeleted)++;
-    //       oldnode->SetFlag(NF_SEEN, false);
-    //       return true;
-    //     }
-
-    oldnode->m_pNodeManager->Undo(oldnode);
-    oldnode->Parent()->m_pNodeManager->Enter(oldnode->Parent());
-
-    // TODO: This is completely the wrong place to trap output
+  while (m_pLastOutput != pLastSeen) {
+    m_pLastOutput->Undo();
+    m_pLastOutput->Leave(); //Should we? I think so, but the old code didn't...?
+    m_pLastOutput->SetFlag(NF_SEEN, false);
+    // TODO: Is this the right place to trap output?
     if(pNumDeleted != NULL)
-      (*pNumDeleted) += oldnode->m_iNumSymbols;
-
-    oldnode->SetFlag(NF_SEEN, false);
-
-    if(oldnode->Parent() != newnode)
-      DeleteCharacters(newnode, oldnode->Parent(), pNumDeleted);
-
-    return true;
-
+      (*pNumDeleted) += m_pLastOutput->m_iNumSymbols;
+    
+    m_pLastOutput = m_pLastOutput->Parent();
+    if (m_pLastOutput) m_pLastOutput->Enter();
   }
-  else {
-    //    std::cout << "DCC" << std::endl;
-    // This one's more complicated - the user may have moved onto a new branch
-    // Find the last seen node on the new branch
-    CDasherNode *lastseen = newnode->Parent();
-
-    while(lastseen != NULL && !(lastseen->GetFlag(NF_SEEN))) {
-      lastseen = lastseen->Parent();
-    };
-    // Delete back to last seen node
-    while(oldnode != lastseen) {
-
-      oldnode->SetFlag(NF_SEEN, false);
-      
-      oldnode->m_pNodeManager->Undo(oldnode);
-
-      if(oldnode->Parent())
-	oldnode->Parent()->m_pNodeManager->Enter(oldnode->Parent());
-
-      if (pNumDeleted != NULL)
-	(*pNumDeleted) += oldnode->m_iNumSymbols;
-
-      oldnode = oldnode->Parent();
-      if(oldnode == NULL) {
-        return false;
-      }
-    }
+  
+  if(!pNewNode->GetFlag(NF_SEEN)) {
+    RecursiveOutput(pNewNode, pAdded);
   }
-  return false;
 }
 
-void CDasherModel::Push_Node(CDasherNode *pNode) {
+void CDasherModel::ExpandNode(CDasherNode *pNode) {
   DASHER_ASSERT(pNode != NULL);
-  // TODO: Fix this and make an assertion again
-  if(pNode->GetFlag(NF_SUBNODE))
-    return;
 
   // TODO: Is NF_ALLCHILDREN any more useful/efficient than reading the map size?
-  
 
   if(pNode->GetFlag(NF_ALLCHILDREN)) {
-    DASHER_ASSERT(pNode->Children().size() > 0);
-    // if there are children just give them a poke
-    CDasherNode::ChildMap::iterator i;
-    for(i = pNode->Children().begin(); i != pNode->Children().end(); i++)
-      (*i)->SetFlag(NF_ALIVE, true);
+    DASHER_ASSERT(pNode->GetChildren().size() > 0);
     return;
   }
 
   // TODO: Do we really need to delete all of the children at this point?
   pNode->Delete_children(); // trial commented out - pconlon
 
-  pNode->SetFlag(NF_ALIVE, true);
-
-  // Populate children creates two levels at once - the groups and their children.
-  pNode->m_pNodeManager->PopulateChildren(pNode);
+#ifdef DEBUG
+  unsigned int iExpect = pNode->ExpectedNumChildren();
+#endif
+  pNode->PopulateChildren();
+#ifdef DEBUG
+  if (iExpect != pNode->GetChildren().size()) {
+    std::cout << "(Note: expected " << iExpect << " children, actually created " << pNode->GetChildren().size() << ")" << std::endl;
+  }
+#endif
 
   pNode->SetFlag(NF_ALLCHILDREN, true);
 
@@ -764,113 +587,38 @@ void CDasherModel::Push_Node(CDasherNode *pNode) {
 
   GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher();
   if(m_bGameMode && pNode->GetFlag(NF_GAME) && pTeacher )
-    {
-      std::string strTargetUtf8Char;
+  {
+    std::string strTargetUtf8Char(pTeacher->GetSymbolAtOffset(pNode->m_iOffset + 1));
       
-      CAlphabetManager::SAlphabetData * pAlphabetData =
-	static_cast<CAlphabetManager::SAlphabetData *>(pNode->m_pUserData);
-      
-      strTargetUtf8Char = pTeacher->GetSymbolAtOffset(pAlphabetData->iOffset+1);
-      
-      CDasherNode::ChildMap::iterator i, j;
-      // Check if this is the last node in the sentence...
-      if(strTargetUtf8Char == "GameEnd")
-	{
-	  pNode->SetFlag(NF_END_GAME, true);
-	  goto multibreak;
-	}
-      // ...if it is not then find which child is next in line.
-      for(i = pNode->Children().begin(); i != pNode->Children().end(); i++)
-	{
-	  // Only look for children who are the same type of node as pNode is (i.e. alphabet)
-	  if((*i)->m_pNodeManager != pNode->m_pNodeManager) continue;
-	  
-	  // Look at the children of the group nodes while we search
-	  if((*i)->GetFlag(NF_SUBNODE))
-	    {
-	      for(j = (*i)->Children().begin(); j != (*i)->Children().end(); j++)
-		{
-		  std::string strNodeUtf8Char;
-		  CDasherNode * pTempNode = (*j);
-		  
-		  pAlphabetData = static_cast<CAlphabetManager::SAlphabetData *>(pTempNode->m_pUserData);
-		  strNodeUtf8Char = m_pNodeCreationManager->GetAlphabet()->GetText(pAlphabetData->iSymbol);
-		  
-		  if(strNodeUtf8Char == strTargetUtf8Char)
-		    {
-		      (*j)->SetFlag(NF_GAME, true);
-		      (*i)->SetFlag(NF_GAME, true);
-		      goto multibreak;
-		    }
-		}
-	    }
-	  else
-	    {
-	      std::string strNodeUtf8Char;
-	      CDasherNode * pTempNode = (*i);
-
-	      pAlphabetData = static_cast<CAlphabetManager::SAlphabetData *>(pTempNode->m_pUserData);
-	      strNodeUtf8Char = m_pNodeCreationManager->GetAlphabet()->GetText(pAlphabetData->iSymbol);
-
-	      if(strNodeUtf8Char == strTargetUtf8Char)
-		{
-		  (*i)->SetFlag(NF_GAME, true);
-		  goto multibreak;
-		}	 
-	    }
-	}
-      // If this line runs then we have a target character which is not in our current alphabet.
-      // So let's give up!
-      pNode->SetFlag(NF_END_GAME, true);
-      
-    multibreak:
-      ; // A lonely semicolon is the null statement - it does nothing.
+    // Check if this is the last node in the sentence...
+    if(strTargetUtf8Char == "GameEnd")
+	    pNode->SetFlag(NF_END_GAME, true);
+	  else if (!pNode->GameSearchChildren(strTargetUtf8Char)) {
+      // Target character not found - not in our current alphabet?!?!
+      // Let's give up!
+      pNode->SetFlag(NF_END_GAME, true); 
     }
+  }
   ////////////////////////////
   
 
 }
 
-void CDasherModel::Recursive_Push_Node(CDasherNode *pNode, int iDepth) {
-  // TODO: is this really useful? Doesn't Push_node itself recurse
-
-  DASHER_ASSERT(pNode != NULL);
-
-  if(pNode->Range() < 0.1 * GetLongParameter(LP_NORMALIZATION)) {
-    return;
-  }
-
-  Push_Node(pNode);
-
-  if(iDepth == 0)
-    return;
-
-  for(unsigned int i(0); i < pNode->ChildCount(); i++) {
-    Recursive_Push_Node(pNode->Children()[i], iDepth - 1);
-  }
-}
-
-bool CDasherModel::RenderToView(CDasherView *pView, bool bRedrawDisplay, SLockData *pLockData) {
-
-  if(pLockData)
-    std::cout << "Rendering locked: " << pLockData->strDisplay << std::endl;
+bool CDasherModel::RenderToView(CDasherView *pView, CExpansionPolicy &policy) {
 
   DASHER_ASSERT(pView != NULL);
   DASHER_ASSERT(m_Root != NULL);
 
-  CDasherNode *pOldNode = Get_node_under_crosshair();
+  // XXX we HandleOutput in RenderToView
+  // DASHER_ASSERT(Get_node_under_crosshair() == m_pLastOutput);
 
-  std::vector<CDasherNode *> vNodeList;
-  std::vector<CDasherNode *> vDeleteList;
-
-  bool bReturnValue;
+  bool bReturnValue = false;
   std::vector<std::pair<myint,bool> > vGameTargetY;
   
   // The Render routine will fill iGameTargetY with the Dasher Coordinate of the 
   // youngest node with NF_GAME set. The model is responsible for setting NF_GAME on
   // the appropriate Nodes.
-  bReturnValue = pView->Render(m_Root, m_Rootmin + m_iTargetOffset, m_Rootmax + m_iTargetOffset, vNodeList, vDeleteList, bRedrawDisplay, &vGameTargetY);
-  
+  pView->Render(m_Root, m_Rootmin + m_iDisplayOffset, m_Rootmax + m_iDisplayOffset, policy, true, &vGameTargetY);  
 
   /////////GAME MODE TEMP//////////////
   if(m_bGameMode)
@@ -878,93 +626,52 @@ bool CDasherModel::RenderToView(CDasherView *pView, bool bRedrawDisplay, SLockDa
       pTeacher->SetTargetY(vGameTargetY);
   //////////////////////////////////////
 
-
-  if(!GetBoolParameter(BP_OLD_STYLE_PUSH)) {
-    for(std::vector<CDasherNode *>::iterator it(vNodeList.begin()); it != vNodeList.end(); ++it)
-      Push_Node(*it);
-  }
-  
-  // TODO: Fix this
-  for(std::vector<CDasherNode *>::iterator it(vDeleteList.begin()); it != vDeleteList.end(); ++it)
-    {
-      if(!(*it)->GetFlag(NF_SUBNODE))
-	(*it)->Delete_children();
-      
-    }
-  
-  CDasherNode *pNewNode = Get_node_under_crosshair();
-
   // TODO: Fix up stats
   // TODO: Is this the right way to handle this?
-  if(pNewNode != pOldNode)
-    HandleOutput(pNewNode, pOldNode, NULL, NULL);
+  HandleOutput(NULL, NULL);
 
-  return bReturnValue;
-}
-
-// Return true to indicate zero or one nodes found, false for more than one
-bool CDasherModel::RecursiveCheckRoot(CDasherNode *pNode, CDasherNode **pNewNode, bool &bFound) {
-  DASHER_ASSERT(pNode != NULL);
-  DASHER_ASSERT(pNewNode != NULL);
-
-  CDasherNode::ChildMap & children = pNode->Children();
+  //ACL Off-screen nodes (zero collapse cost) will have been collapsed already.
+  //Hence, this acts to maintain the node budget....or whatever the queue's policy is!
+  if (policy.apply(m_pNodeCreationManager,this)) bReturnValue=true;
   
-  for(CDasherNode::ChildMap::iterator it(children.begin()); it != children.end(); ++it) {
-    if((*it)->GetFlag(NF_SUBNODE)) {
-      if(!RecursiveCheckRoot(*it, pNewNode, bFound))
-	return false;
-    }
-    else if((*it)->GetFlag(NF_SUPER)) {
-      if(bFound) // TODO: This should be an error (and probably isn't worth checking for)
-	return false;
-      else {
-	*pNewNode = *it;
-	bFound = true;
-      }
-    }
-  }
-
-  return true;
+  return bReturnValue;
 }
 
 bool CDasherModel::CheckForNewRoot(CDasherView *pView) {
   DASHER_ASSERT(m_Root != NULL);
   // TODO: pView is redundant here
 
+#ifdef DEBUG
   CDasherNode *pOldNode = Get_node_under_crosshair();
+#endif
 
   CDasherNode *root(m_Root);
 
   if(!(m_Root->GetFlag(NF_SUPER))) {
     Reparent_root(root->Lbnd(), root->Hbnd());
-    DASHER_ASSERT(!(m_Root->GetFlag(NF_SUBNODE)));
     return(m_Root != root);
   }
 
-  DASHER_ASSERT(!(m_Root->GetFlag(NF_SUBNODE)));
-
   CDasherNode *pNewRoot = NULL;
 
-  bool bFound = false;
-
-  if(RecursiveCheckRoot(m_Root, &pNewRoot, bFound)) {
-    // TODO: I think this if statement is reduncdent, return value of above is always equal to bFound
-    // not true - pconlon
-    ////GAME MODE TEMP - only change the root if it is on the game path/////////
-    if(bFound && (!m_bGameMode || pNewRoot->GetFlag(NF_GAME))) {
-      m_Root->DeleteNephews(pNewRoot);
-      RecursiveMakeRoot(pNewRoot);
+  for (CDasherNode::ChildMap::const_iterator it = m_Root->GetChildren().begin(); it != m_Root->GetChildren().end(); it++) {
+    if ((*it)->GetFlag(NF_SUPER)) {
+      //at most one child should have NF_SUPER set...
+      DASHER_ASSERT(pNewRoot == NULL);
+      pNewRoot = *it;
+#ifndef DEBUG
+      break;
+#endif
     }
   }
+  ////GAME MODE TEMP - only change the root if it is on the game path/////////
+  if (pNewRoot && (!m_bGameMode || pNewRoot->GetFlag(NF_GAME))) {
+    m_Root->DeleteNephews(pNewRoot);
+    RecursiveMakeRoot(pNewRoot);
+  }
 
-  CDasherNode *pNewNode = Get_node_under_crosshair();
+  DASHER_ASSERT(Get_node_under_crosshair() == pOldNode);
 
-  // TODO: Fix this, make more efficient
-  // TODO: Unnecessary? Should never change anuything
-  if(pNewNode != pOldNode)
-    HandleOutput(pNewNode, pOldNode, NULL, NULL);
-
-  DASHER_ASSERT(!(m_Root->GetFlag(NF_SUBNODE)));
   return false;
 }
 
@@ -987,15 +694,9 @@ void CDasherModel::ScheduleZoom(dasherint X, dasherint Y, int iMaxZoom)
   dasherint y1 = Y - X;     // y =  x + (Y - X)
   dasherint y2 = Y + X;     // y = -x + (Y + X)
 
-  // The maximum linear region of the y-axis is from 5% - 95% of ymax
-  // 5% and 95% come from dY3, dY2 in CDasherViewSquare::Cymap::Cymap,
-  // and non-linearity between dasher coords m_Y3 to m_Y2 in
-  // CDasherViewSquare::Cymap::map
-  // (It is good to test with Y1 = 0, Y2 = ymax.)
-  // XXX PRLW But m_Y3, m_Y2 are out of scope...
-  const dasherint Y1 = ( 5 * scale) / 100;  // m_Y3;
-  const dasherint Y2 = (95 * scale) / 100;  // m_Y2;
   // Rename for readability.
+  const dasherint Y1 = 0;
+  const dasherint Y2 = ymax;
   const dasherint R1 = m_Rootmin;
   const dasherint R2 = m_Rootmax;
 
@@ -1051,16 +752,15 @@ void CDasherModel::Offset(int iOffset) {
   m_Rootmin += iOffset;
   m_Rootmax += iOffset;
 
-  m_iTargetOffset -= iOffset;
+  if (GetBoolParameter(BP_SMOOTH_OFFSET))
+    m_iDisplayOffset -= iOffset;
 } 
 
-void CDasherModel::MatchTarget(bool bReverse) {
-  // TODO: Does anything need to happen wrt bReverse here?
+void CDasherModel::AbortOffset() {
+  m_Rootmin += m_iDisplayOffset;
+  m_Rootmax += m_iDisplayOffset;
 
-  m_Rootmin += m_iTargetOffset;
-  m_Rootmax += m_iTargetOffset;
-
-  m_iTargetOffset = 0;
+  m_iDisplayOffset = 0;
 }
 
 void CDasherModel::LimitRoot(int iMaxWidth) {
@@ -1088,5 +788,5 @@ void CDasherModel::SetControlOffset(int iOffset) {
   // work right now.
   CDasherNode *pNode = Get_node_under_crosshair();
   
-  pNode->m_pNodeManager->SetControlOffset(pNode, iOffset);
+  pNode->SetControlOffset(iOffset);
 }
