@@ -1,6 +1,6 @@
 // DasherModel.h
 //
-// Copyright (c) 2007 The Dasher Team
+// Copyright (c) 2008 The Dasher Team
 //
 // This file is part of Dasher.
 //
@@ -21,8 +21,8 @@
 #ifndef __DasherModel_h__
 #define __DasherModel_h__
 
-#ifndef WIN32
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
 #endif
 
 #include <climits>
@@ -36,6 +36,7 @@
 #include "DasherTypes.h"
 #include "FrameRate.h"
 #include "NodeCreationManager.h"
+#include "ExpansionPolicy.h"
 
 namespace Dasher {
   class CDasherModel;
@@ -55,7 +56,7 @@ namespace Dasher {
 ///             knows the current viewpoint
 ///             knows how to evolve the viewpoint
 ///
-class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
+class Dasher::CDasherModel:public CFrameRate, private NoClones
 {
  public:
 
@@ -73,12 +74,9 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   /// @{
 
   ///
-  /// Update the root location - called in response to regular timer
-  /// callbacks, mainly from the tier events of various button
-  /// handlers
-  /// TODO: Make this a bit more central in the button handler hierarchy
-  ///
-  bool UpdatePosition(myint, myint, unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB* pAdded = NULL, int* pNumDeleted = NULL);  
+  /// Update the root location with *one step* towards the specified
+  /// co-ordinates - used by timer callbacks (for non-button modes)
+  void OneStepTowards(myint, myint, unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB* pAdded = NULL, int* pNumDeleted = NULL);  
 
   ///
   /// Notify the framerate class that a new frame has occurred
@@ -93,52 +91,14 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   void Offset(int iOffset);
  
   ///
-  /// Reset the 'target' root coordinates to match those currently visible. 
-  /// Appropriate for abrubt changes in behaviour (such as backing off in 
-  /// button modes)
-  /// 
-  void MatchTarget(bool bReverse);
+  /// Make the 'target' root coordinates match those currently visible, so any
+  /// Offset(int) currently in progress (i.e. being smoothed over several
+  /// frames) stops (at whatever point it's currently reached). Appropriate for
+  /// abrupt changes in behaviour (such as backing off in  button modes)
+  void AbortOffset();
 
   /// @}
 
-  ///
-  /// Get the current framerate
-  ///
-
-  double Framerate() const {
-    return m_fr.Framerate();
-  }
-
-  ///
-  /// Reset the framerate class
-  /// TODO: Need to check semantics here
-  /// Called from CDasherInterfaceBase::UnPause;
-  ///
-
-  void Reset_framerate(unsigned long Time) {
-    m_fr.Reset(Time);
-  }
-
-  ///
-  /// Initialise the framerate class - presumably called whenever
-  /// dasher is stoppe, but the actual semantics here need to be
-  /// verified
-  /// Called from CDasherInterfacebase::Halt 
-  ///
-
-  void Halt() {
-    m_fr.Initialise();
-  }
-
-  ///
-  /// Set the target bitrate - probably shouldn't be called externally
-  /// - could implement through the event subsystem instead
-  ///
-  
-/*   void SetBitrate(double TargetRate) { */
-/*     m_fr.SetBitrate(TargetRate); */
-/*   }  */
-
   ///
   /// Reset counter of total nats entered
   ///
@@ -161,10 +121,12 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   /// @{
 
   /// 
-  /// Render the model to a given view
+  /// Render the model to a given view. Return if any nodes were
+  /// expanded, as if so we may want to render *another* frame to
+  /// perform further expansion.
   ///
 
-  bool RenderToView(CDasherView *pView, bool bRedrawDisplay, SLockData *pLockData);
+  bool RenderToView(CDasherView *pView, CExpansionPolicy &policy);
 
   /// @}
 
@@ -181,12 +143,11 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   void ScheduleZoom(dasherint iDasherX, dasherint iDasherY, int iMaxZoom = 0);
 
   ///
-  /// Return the number of remaining zoom steps
+  /// Update the bounds of the root node for the next step in any
+  /// still-in-progress zoom scheduled by ScheduleZoom (does nothing
+  /// if no steps remaining / no zoom scheduled).
   ///
-
-  int ScheduledSteps() {
-    return m_deGotoQueue.size();
-  }
+  bool NextScheduledStep(unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB* pAdded = NULL, int *pNumDeleted = NULL);
 
   /// @}
 
@@ -207,19 +168,10 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
     m_iStartTime = 0;
   };
 
-  ///
-  /// Return whether Dasher is currently being temporarily slowed or
-  /// not.
-  ///
-
-  bool IsSlowdown(unsigned long iTime) {
-    return ((iTime - m_iStartTime) < static_cast<unsigned long>(GetLongParameter(LP_SLOW_START_TIME)));
-  };
-
   ///
   /// Check whether a change of root node is needed, and perform the
   /// update if so
-  /// TODO: Could be done in UpdatePosition?
+  /// TODO: Could be done in UpdateBounds?
   ///
 
   bool CheckForNewRoot(CDasherView *pView);
@@ -234,23 +186,30 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   void SetOffset(int iLocation, CDasherView *pView);
 
   ///
-  /// TODO: Implement this
+  /// TODO: Figure out how all these "offset"s work / relate to each other - if they do! In particular,
+  /// what do we need DasherModel's own m_iOffset (which measures in _bytes_, not unicode characters!) for?
   ///
 
   int GetOffset() {
-    return 0;
+    return m_pLastOutput->m_iOffset+1;
   };
 
+  /// Create the children of a Dasher node
+  void ExpandNode(CDasherNode * pNode); 
+  
   void SetControlOffset(int iOffset);
 
  private:
 
+  /// Common portion of OneStepTowards / NextScheduledStep, taking
+  /// bounds for the root node in the next frame.
+  void UpdateBounds(myint iNewMin, myint iNewMax, unsigned long iTime, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted);
+
   /// Struct representing intermediate stages in the goto queue
   ///
   struct SGotoItem {
     myint iN1;
     myint iN2;
-    int iStyle;
   };
   
   // Pointers to various auxilliary objects
@@ -279,7 +238,7 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   // Offset used when presenting the model to the user, specified as
   // Displayed rootmin/max - actual rootmin/rootmax
-  myint m_iTargetOffset; 
+  myint m_iDisplayOffset;
 
   CDasherNode *m_pLastOutput;
 
@@ -305,12 +264,13 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   // Model status...
 
-  // Helper class to estimate frame rate
-  CFrameRate m_fr;  
 
   // Time at which the model was started (ie last unpaused, used for gradual speed up)
   // TODO: Implementation is very hacky at the moment
-  // TODO: Duplicates functionality previously implemented elsewhere
+  // TODO: Duplicates functionality previously implemented elsewhere...
+  //   ...ACL 22/5/09 does it? There was an even hackier implementation, of resetting the
+  //   framerate, used for control mode (ControlManager.cpp), but that's all I could find
+  //   - and that seemed even worse, so I've removed it in favour of this here....?
   unsigned long m_iStartTime;
   
   // Offset into buffer of node currently under crosshair
@@ -321,10 +281,6 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   // Information entered so far in this model
   double m_dTotalNats; 
 
-  // Number of nodes rendered
-  int m_iRenderCount;
-
-
 
   CDasherNode *Get_node_under_mouse(myint smousex, myint smousey);
 
@@ -348,46 +304,12 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   void Get_new_root_coords(myint mousex, myint mousey, myint &iNewMin, myint &iNewMax, unsigned long iTime);
 
 
-  /// Called from ScheduleZoom
-  double CorrectionFactor(int dasherx, int dashery);
-
   /// Should be public?
   void InitialiseAtOffset(int iOffset, CDasherView *pView);
 
   /// Called from InitialiseAtOffset
   void DeleteTree();
 
-  /// Push nodes recursively (d'oh!) - seems to be a little broken at the moment, need to check where this gets called
-  /// TODO: Need to sort out, need to check that this isn't generating too many nodes
-  void Recursive_Push_Node(CDasherNode * pNode, int depth);
-
-  /// Create the children of a Dasher node
-  void Push_Node(CDasherNode * pNode); 
-
-
-  ///
-  /// Perform output on a node - recurse up the tree outputting any
-  /// symbols which have not been output so far. Neeed to check
-  /// behaviour with respect to deletion
-  ///
-
-  void RecursiveOutput(CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* pAdded);
-
-
-  ///
-  /// Check semantics here
-  ///
-
-  void OutputCharacters(CDasherNode * node);
-  bool DeleteCharacters(CDasherNode * newnode, CDasherNode * oldnode, int* pNumDeleted = NULL);
-
-  /// 
-  /// Old style drilling down of nodes - optionally can still be
-  /// called
-  ///
-
-  void OldPush(myint iMousex, myint iMousey);
-
   /// 
   /// Make a child of the root into a new root
   ///
@@ -402,12 +324,14 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
   void RecursiveMakeRoot(CDasherNode *pNewRoot);
 
   ///
-  /// Rebuild the data structure such that a given node is guaranteed
-  /// to be in the same place on the screen. This would usually be the
-  /// node under the crosshair
+  /// Makes the node under the crosshair the root by deleting everything
+  /// outside it, then rebuilds the nodes beneath it. (Thus, the node under
+  /// the crosshair stays in the same place.) Used when control mode is turned
+  /// on or off, or more generally, when the sizes of child nodes may have
+  /// changed.
   ///
 
-  void RebuildAroundNode(CDasherNode *pNode);
+  void RebuildAroundCrosshair();
 
   ///
   /// Rebuild the parent of the current root - used during backing off
@@ -417,19 +341,25 @@ class Dasher::CDasherModel:public Dasher::CDasherComponent, private NoClones
 
   ///
   /// Return a pointer to the Dasher node which is currently under the
-  /// crosshair. Apparently this is needed for game mode.
+  /// crosshair. Used for output, and apparently needed for game mode.
   ///
 
   CDasherNode *Get_node_under_crosshair();    
 
   ///
-  /// Handle the output caused by a change in node being over the
-  /// crosshair
+  /// Output a node, which has not been seen (& first, any ancestors that haven't been seen either),
+  /// but which _is_ a descendant of m_pLastOutput.
+  ///
+  
+  void RecursiveOutput(CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* pAdded);
+  
+  ///
+  /// Handle the output caused by a change in node over the crosshair. Specifically,
+  /// deletes from m_pLastOutput back to closest ancestor of pNewNode,
+  /// then outputs from that ancestor to the node now under the crosshair (inclusively)
   ///
 
-  void HandleOutput(CDasherNode *pNewNode, CDasherNode *pOldNode, Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted);
-
-  bool RecursiveCheckRoot(CDasherNode *pNode, CDasherNode **pNewNode, bool &bFound);
+  void HandleOutput(Dasher::VECTOR_SYMBOL_PROB* pAdded, int* pNumDeleted);
 
 
   ///
