@@ -71,13 +71,18 @@ public:
   /// 
   /// Convert a screen co-ordinate to Dasher co-ordinates
   ///
-  void Screen2Dasher(screenint iInputX, screenint iInputY, myint & iDasherX, myint & iDasherY,bool b1D, bool bNonlinearity);
+  void Screen2Dasher(screenint iInputX, screenint iInputY, myint & iDasherX, myint & iDasherY);
 
   ///
   /// Convert Dasher co-ordinates to screen co-ordinates
   ///
   void Dasher2Screen(myint iDasherX, myint iDasherY, screenint & iScreenX, screenint & iScreenY);
 
+  ///
+  /// Convert Dasher co-ordinates to polar co-ordinates (r,theta), with 0<r<1, 0<theta<2*pi
+  ///
+  virtual void Dasher2Polar(myint iDasherX, myint iDasherY, double &r, double &theta);	
+	
   /// 
   /// Return true if a node spanning y1 to y2 is entirely enclosed by
   /// the screen boundary
@@ -89,11 +94,6 @@ public:
   ///
   void VisibleRegion( myint &iDasherMinX, myint &iDasherMinY, myint &iDasherMaxX, myint &iDasherMaxY );
 
-  double xmap(double x) const;
-  double ymap(double x) const {
-    return m_ymap.map( (myint)x );
-  };
-
   /// @}
 
 
@@ -109,15 +109,17 @@ private:
   ///
   /// Render the current state of the model.
   ///
-
-  virtual void RenderNodes(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, std::vector<std::pair<myint,bool> > *pvGamePointer);
-
+  virtual void RenderNodes(CDasherNode *pRoot, myint iRootMin, myint iRootMax, CExpansionPolicy &policy, std::vector<std::pair<myint,bool> > *pvGamePointer);
   
   ///
   /// Recursively render all nodes in a tree. Responsible for all the Render_node calls
   ///
 
-  bool RecursiveRender(CDasherNode * Render, myint y1, myint y2, int mostleft, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, std::vector<std::pair<myint,bool> > *pvGamePointer, bool bDraw,myint parent_width,int parent_color, int iDepth);
+  void RecursiveRender(CDasherNode * Render, myint y1, myint y2, int mostleft, CExpansionPolicy &policy, double dMaxCost, std::vector<std::pair<myint,bool> > *pvGamePointer, myint parent_width,int parent_color, int iDepth);
+
+  ///Check that a node is large enough, and onscreen, to render;
+  ///calls RecursiveRender if so, or collapses the node immediately if not
+  bool CheckRender(CDasherNode * Render, myint y1, myint y2, int mostleft, CExpansionPolicy &policy, double dMaxCost, std::vector<std::pair<myint,bool> > *pvGamePointer, myint parent_width,int parent_color, int iDepth);
 
   /// Render a single node
   /// \param Color The colour to draw it
@@ -134,7 +136,6 @@ private:
 
   int RenderNodeOutlineFast(const int Color, myint y1, myint y2, int &mostleft, const std::string &sDisplayText, bool bShove); 
   int RenderNodePartFast(const int Color, myint y1, myint y2, int &mostleft, const std::string &sDisplayText, bool bShove, myint iParentWidth);
-  int RenderNodeFatherFast(const int parent_color, myint y1, myint y2, int &mostleft, const std::string &sDisplayText, bool bShove,myint iParentWidth, bool bVisible);
 #ifdef _WIN32
   ///
   /// FIXME - couldn't find windows version of round(double) so here's one!
@@ -151,7 +152,7 @@ private:
 #endif
 
   // Class definitions
-
+  ///Implements the non-linearity near the extremes of the Y axis
   class Cymap {
   public:
     Cymap(myint iScale);
@@ -162,6 +163,7 @@ private:
     myint m_Y1, m_Y2, m_Y3;
   };
 
+  double xmap(double x) const;
   double xmax(double x, double y) const;
   double ixmap(double x) const;
   inline void Crosshair(myint sx);
@@ -186,6 +188,7 @@ private:
  
   double m_dXmpa, m_dXmpb, m_dXmpc, m_dXmpd;
   screenint CanvasX, CanvasY, CanvasBorder;
+  screenint iCenterX;
   Cymap m_ymap;
 
   // Cached values for scaling
