@@ -3,11 +3,12 @@
 
 #include "DefaultFilter.h"
 
+namespace Dasher {
 /// \ingroup InputFilter
 /// @{
 class CEyetrackerFilter : public CDefaultFilter {
  public:
-  CEyetrackerFilter(Dasher::CEventHandler * pEventHandler, CSettingsStore *pSettingsStore, CDasherInterfaceBase *pInterface, CDasherModel *m_pDasherModel);
+  CEyetrackerFilter(Dasher::CEventHandler * pEventHandler, CSettingsStore *pSettingsStore, CDasherInterfaceBase *pInterface);
 
   bool GetSettings(SModuleSettings **pSettings, int *iCount);
 
@@ -24,6 +25,7 @@ class CEyetrackerFilter : public CDefaultFilter {
   int m_iCounter;
   int m_iFilterTimescale;
 };
+}
 /// @}
 
 #endif
