@@ -3,22 +3,25 @@
 
 #include "InputFilter.h"
 
+namespace Dasher {
+/// \ingroup InputFilter
+/// @{
 class COneButtonFilter : public CInputFilter {
  public:
   COneButtonFilter(Dasher::CEventHandler * pEventHandler, CSettingsStore *pSettingsStore, CDasherInterfaceBase *pInterface);
   ~COneButtonFilter();
 
-  virtual void HandleEvent(Dasher::CEvent * pEvent);
-
   virtual bool DecorateView(CDasherView *pView);
-  virtual void Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel);
-  virtual void KeyDown(int iTime, int iId, CDasherModel *pModel);
-  
+  virtual bool Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted, CExpansionPolicy **pol);
+  virtual void KeyDown(int iTime, int iId, CDasherView *pView, CDasherModel *pModel, CUserLogBase *pUserLog);
+  bool GetSettings(SModuleSettings **pSettings, int *iCount);
  private:
 
   bool bStarted;
   int iStartTime;
   int iLocation;
 };
+}
+/// @}
 
 #endif
