@@ -3,7 +3,9 @@
 
 #include "DasherInterfaceBase.h"
 #include "DasherComponent.h"
-
+namespace Dasher {
+/// \defgroup Start Start handlers
+/// @{
 class CStartHandler : public CDasherComponent {
 public:
   CStartHandler(Dasher::CEventHandler * pEventHandler, CSettingsStore * pSettingsStore, CDasherInterfaceBase *pInterface) 
@@ -18,5 +20,7 @@ public:
 protected:
   CDasherInterfaceBase *m_pInterface;
 };
+}
+/// @}
 
 #endif
