@@ -21,16 +21,16 @@
 #ifndef __TWO_BUTTON_DYNAMIC_FILTER_H__
 #define __TWO_BUTTON_DYNAMIC_FILTER_H__
 
-#include "DynamicFilter.h"
+#include "ButtonMultiPress.h"
 
 #include <deque>
 
+namespace Dasher {
 /// \ingroup InputFilter
 /// @{
-class CTwoButtonDynamicFilter : public CDynamicFilter {
+class CTwoButtonDynamicFilter : public CButtonMultiPress {
  public:
   CTwoButtonDynamicFilter(Dasher::CEventHandler * pEventHandler, CSettingsStore *pSettingsStore, CDasherInterfaceBase *pInterface);
-  ~CTwoButtonDynamicFilter();
 
   // Inherited methods
   virtual bool DecorateView(CDasherView *pView);
@@ -40,44 +40,28 @@ class CTwoButtonDynamicFilter : public CDynamicFilter {
 
   virtual bool GetSettings(SModuleSettings **pSettings, int *iCount);
 
- 
+  virtual bool GetMinWidth(int &iMinWidth);
+
+  virtual void HandleEvent(Dasher::CEvent *pEvent);
   
+ protected:
+  virtual void run();
+  virtual void pause();
+  virtual void reverse();
+
+  //override to inspect x,y coords of mouse clicks/taps
+  virtual void KeyDown(int Time, int iId, CDasherView *pDasherView, CDasherModel *pModel, CUserLogBase *pUserLog, bool bPos, int iX, int iY);
+  virtual void KeyUp(int Time, int iId, CDasherView *pDasherView, CDasherModel *pModel, bool bPos, int iX, int iY);
+	
  private:
-  virtual bool TimerImpl(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted);
+  unsigned int maxClickCount() {return GetBoolParameter(BP_2B_INVERT_DOUBLE) ? 3 : 2;}
+  virtual bool TimerImpl(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted, CExpansionPolicy **pol);
   virtual void ActionButton(int iTime, int iButton, int iType, CDasherModel *pModel, CUserLogBase *pUserLog);
+  double m_dLagMul;
 
-  virtual void RevertPresses(int iCount);
-
-  void AutoSpeedSample(int iTime, CDasherModel *pModel);
-  void AutoSpeedUndo(int iCount);
-
-  class SBTree {
-  public:
-    SBTree(int iValue);
-    ~SBTree();
-
-    void Add(int iValue);
-    SBTree* Delete(int iValue);
-
-    int GetCount() {
-      return m_iCount;
-    };
-
-    int GetOffset(int iOffset);
-
-    void SetRightMost(SBTree* pNewTree);
-
-  private:
-    int m_iValue;
-    SBTree *m_pLeft;
-    SBTree *m_pRight;
-    int m_iCount;
-  };
-
-  int m_iLastTime;
-  SBTree *m_pTree;
-  std::deque<int> m_deOffsetQueue;
+  void ApplyOffset(CDasherModel *pModel, long lOffset);
 };
+}
 /// @}
 
 #endif
