@@ -29,15 +29,15 @@ CCanvas::CCanvas(GtkWidget *pCanvas, CPangoCache *pPangoCache)
 
   m_pDisplaySurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, m_iWidth, m_iHeight);
   m_pDecorationSurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, m_iWidth, m_iHeight);
-  m_pOnscreenSurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, m_iWidth, m_iHeight);
+  //m_pOnscreenSurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, m_iWidth, m_iHeight);
 
 #else
 
-  m_pDummyBuffer = gdk_pixmap_new(pCanvas->window, m_iWidth, m_iHeight, -1);
+  //m_pDummyBuffer = gdk_pixmap_new(pCanvas->window, m_iWidth, m_iHeight, -1);
 
   m_pDisplayBuffer = gdk_pixmap_new(pCanvas->window, m_iWidth, m_iHeight, -1);
   m_pDecorationBuffer = gdk_pixmap_new(pCanvas->window, m_iWidth, m_iHeight, -1);
-  m_pOnscreenBuffer = gdk_pixmap_new(pCanvas->window, m_iWidth, m_iHeight, -1);
+  //m_pOnscreenBuffer = gdk_pixmap_new(pCanvas->window, m_iWidth, m_iHeight, -1);
 
   // Set the display buffer to be current
 
@@ -59,9 +59,7 @@ CCanvas::CCanvas(GtkWidget *pCanvas, CPangoCache *pPangoCache)
   cairo_set_line_cap(cr, CAIRO_LINE_CAP_SQUARE);
   cairo_set_line_width(cr, 1.0);
 
-  onscreen_cr = cairo_create(m_pOnscreenSurface);
-
-  widget_cr = gdk_cairo_create(m_pCanvas->window) ;
+  //onscreen_cr = cairo_create(m_pOnscreenSurface);
 
 #endif
 
@@ -76,16 +74,15 @@ CCanvas::~CCanvas() {
   cr = NULL;
   cairo_surface_destroy(m_pDisplaySurface);
   cairo_surface_destroy(m_pDecorationSurface);
-  cairo_surface_destroy(m_pOnscreenSurface);  
+  //cairo_surface_destroy(m_pOnscreenSurface);  
   cairo_destroy(display_cr);
   cairo_destroy(decoration_cr);
-  cairo_destroy(onscreen_cr);
-  cairo_destroy(widget_cr);
+  //cairo_destroy(onscreen_cr);
 #else
-  g_object_unref(m_pDummyBuffer);
+  //g_object_unref(m_pDummyBuffer);
   g_object_unref(m_pDisplayBuffer);
   g_object_unref(m_pDecorationBuffer);
-  g_object_unref(m_pOnscreenBuffer);
+  //g_object_unref(m_pOnscreenBuffer);
 #endif
 
 #if WITH_CAIRO
@@ -122,16 +119,12 @@ void CCanvas::Display() {
   // FIXME - Some of this stuff is probably not needed
   //  GdkRectangle update_rect;
 
-  GdkGC *graphics_context;
 #if WITH_CAIRO
 #else
+  GdkGC *graphics_context;
   GdkColormap *colormap;
-#endif
 
   graphics_context = m_pCanvas->style->fg_gc[GTK_WIDGET_STATE(m_pCanvas)];
-
-#if WITH_CAIRO
-#else
   colormap = gdk_colormap_get_system();
 #endif
 
@@ -173,23 +166,26 @@ void CCanvas::Display() {
   //  gdk_window_begin_paint_rect(m_pCanvas->window, &sRect);
 
 #if WITH_CAIRO  
+  cairo_t *widget_cr;
+  widget_cr = gdk_cairo_create(m_pCanvas->window);
   cairo_set_source_surface(widget_cr, m_pDecorationSurface, 0, 0);
   cairo_rectangle(widget_cr, 0, 0, m_iWidth, m_iHeight);
   cairo_fill(widget_cr);
+  cairo_destroy(widget_cr);
 #else
   gdk_draw_drawable(m_pCanvas->window, m_pCanvas->style->fg_gc[GTK_WIDGET_STATE(m_pCanvas)], m_pDecorationBuffer, 0, 0, 0, 0, m_iWidth, m_iHeight);
 #endif
 
   //   gdk_window_end_paint(m_pCanvas->window);
 
-  // Restore original graphics context (?)
+  // Restore original graphics context (colours)
 
   END_DRAWING;
 
   // gtk_main_iteration_do(0);
 }
 
-void CCanvas::DrawRectangle(int x1, int y1, int x2, int y2, int Color, int iOutlineColour, Opts::ColorSchemes ColorScheme, bool bDrawOutline, bool bFill, int iThickness) {
+void CCanvas::DrawRectangle(screenint x1, screenint y1, screenint x2, screenint y2, int Color, int iOutlineColour, Opts::ColorSchemes ColorScheme, bool bDrawOutline, bool bFill, int iThickness) {
 
   //  std::cout << "Raw Rectangle, (" << x1 << ", " << y1 << ") - (" << x2 << ", " << y2 << ")" << std::endl;
 
@@ -302,7 +298,7 @@ void CCanvas::DrawCircle(screenint iCX, screenint iCY, screenint iR, int iColour
 
 void CCanvas::Polygon(Dasher::CDasherScreen::point *Points, int Number, int Colour, int iWidth) {
 
-  if(iWidth == 1) // This is to make it work propely on Windows
+  if(iWidth == 1) // This is to make it work properly on Windows
     iWidth = 0; 
 
 #if WITH_CAIRO
@@ -383,7 +379,7 @@ void CCanvas::Polyline(Dasher::CDasherScreen::point *Points, int Number, int iWi
   END_DRAWING;
 }
 
-void CCanvas::DrawString(const std::string &String, int x1, int y1, int size) {
+void CCanvas::DrawString(const std::string &String, screenint x1, screenint y1, int size) {
   
 #if WITH_CAIRO
 #else
@@ -415,8 +411,7 @@ void CCanvas::DrawString(const std::string &String, int x1, int y1, int size) {
   END_DRAWING;
 }
 
-void CCanvas::TextSize(const std::string &String, int *Width, int *Height, int size) {
-
+void CCanvas::TextSize(const std::string &String, screenint *Width, screenint *Height, int size) {
 #if WITH_CAIRO
   PangoLayout *pLayout(m_pPangoCache->GetLayout(cr, String, size));
 #else
@@ -442,17 +437,13 @@ void CCanvas::SendMarker(int iMarker) {
   case 1: // Switch to decorations buffer
 
 #if WITH_CAIRO
-    BEGIN_DRAWING;
     cairo_set_source_surface(decoration_cr, m_pDisplaySurface, 0, 0);
     cairo_rectangle(decoration_cr, 0, 0, m_iWidth, m_iHeight);
     cairo_fill(decoration_cr);
-    END_DRAWING;
+    cr = decoration_cr;
 #else
     gdk_draw_drawable(m_pDecorationBuffer, m_pCanvas->style->fg_gc[GTK_WIDGET_STATE(m_pCanvas)], m_pDisplayBuffer, 0, 0, 0, 0, m_iWidth, m_iHeight);
     m_pOffscreenBuffer = m_pDecorationBuffer;
-#endif
-#if WITH_CAIRO
-    cr = decoration_cr;
 #endif
     break;
   }
@@ -464,7 +455,7 @@ void CCanvas::SetColourScheme(const CColourIO::ColourInfo *pColourScheme) {
 #if WITH_CAIRO
   if (cairo_colours)
     delete[] cairo_colours;
-  cairo_colours = new my_cairo_colour_t[iNumColours];
+  cairo_colours = new cairo_pattern_t*[iNumColours];
 #else
   if (colours)
     delete[] colours;
@@ -473,9 +464,11 @@ void CCanvas::SetColourScheme(const CColourIO::ColourInfo *pColourScheme) {
 
   for(int i = 0; i < iNumColours; i++) {
 #if WITH_CAIRO
-    cairo_colours[i].r = pColourScheme->Reds[i] / 255.0;
-    cairo_colours[i].g = pColourScheme->Greens[i] / 255.0;
-    cairo_colours[i].b = pColourScheme->Blues[i] / 255.0;
+    cairo_colours[i] = cairo_pattern_create_rgb (
+      pColourScheme->Reds[i]   / 255.0,
+      pColourScheme->Greens[i] / 255.0,
+      pColourScheme->Blues[i]  / 255.0
+    );
 #else
     colours[i].pixel=0;
     colours[i].red=pColourScheme->Reds[i]*257;
