@@ -1,11 +1,14 @@
 #include "../Common/Common.h"
-#include "../../config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
+#include <cstring>
 #include <iostream>
 #include "DasherControl.h"
 #include "Timer.h"
 #include "../DasherCore/Event.h"
-#include "../DasherCore/WrapperFactory.h"
+#include "../DasherCore/ModuleManager.h"
 
 #include <fcntl.h>
 
@@ -13,6 +16,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 #include <sys/stat.h>
+#include <unistd.h>
 using namespace std;
 
 // 'Private' methods (only used in this file)
@@ -41,36 +45,31 @@ CDasherControl::CDasherControl(GtkVBox *pVBox, GtkDasherControl *pDasherControl)
   //  m_pKeyboardHelper->Grab(GetBoolParameter(BP_GLOBAL_KEYBOARD));
 }
 
-void CDasherControl::CreateLocalFactories() {
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherMouseInput(m_pEventHandler, m_pSettingsStore)));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CSocketInput(m_pEventHandler, m_pSettingsStore)));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasher1DMouseInput(m_pEventHandler, m_pSettingsStore)));
+void CDasherControl::CreateModules() {
+  CDasherInterfaceBase::CreateModules(); //create default set first
+  // Create locally cached copies of the mouse input objects, as we
+  // need to pass coordinates to them from the timer callback
+  m_pMouseInput =
+    (CDasherMouseInput *)  RegisterModule(new CDasherMouseInput(m_pEventHandler, m_pSettingsStore));
+  SetDefaultInputDevice(m_pMouseInput);
+  m_p1DMouseInput =
+    (CDasher1DMouseInput *)RegisterModule(new CDasher1DMouseInput(m_pEventHandler, m_pSettingsStore));
+  RegisterModule(new CSocketInput(m_pEventHandler, m_pSettingsStore));
 
 #ifdef JOYSTICK
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherJoystickInput(m_pEventHandler, m_pSettingsStore, this)));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherJoystickInputDiscrete(m_pEventHandler, m_pSettingsStore, this)));
+  RegisterModule(new CDasherJoystickInput(m_pEventHandler, m_pSettingsStore, this));
+  RegisterModule(new CDasherJoystickInputDiscrete(m_pEventHandler, m_pSettingsStore, this));
 #endif
   
 #ifdef TILT
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherTiltInput(m_pEventHandler, m_pSettingsStore, this)));
+  RegisterModule(new CDasherTiltInput(m_pEventHandler, m_pSettingsStore, this));
 #endif
-  
-  
-  // Create locally cached copies of the mouse input objects, as we
-  // need to pass coordinates to them from the timer callback
-  
-  m_pMouseInput = (CDasherMouseInput *)GetModule(0);
-  m_pMouseInput->Ref();
-
-  m_p1DMouseInput = (CDasher1DMouseInput *)GetModule(2);
-  m_p1DMouseInput->Ref();
-
 }
 
 void CDasherControl::SetupUI() {
   m_pCanvas = gtk_drawing_area_new();
   GTK_WIDGET_SET_FLAGS(m_pCanvas, GTK_CAN_FOCUS);
-  gtk_widget_set_double_buffered(m_pCanvas, false);
+  GTK_WIDGET_UNSET_FLAGS(m_pCanvas, GTK_DOUBLE_BUFFERED);
 
   GtkWidget *pFrame = gtk_frame_new(NULL);
   gtk_frame_set_shadow_type(GTK_FRAME(pFrame), GTK_SHADOW_IN); 
@@ -109,7 +108,7 @@ void CDasherControl::SetupUI() {
 void CDasherControl::SetupPaths() {
   char *home_dir;
   char *user_data_dir;
-  char *system_data_dir;
+  const char *system_data_dir;
 
   home_dir = getenv("HOME");
   user_data_dir = new char[strlen(home_dir) + 10];
@@ -190,12 +189,10 @@ void CDasherControl::ScanColourFiles(std::vector<std::string> &vFileList) {
 
 CDasherControl::~CDasherControl() {
   if(m_pMouseInput) {
-    m_pMouseInput->Unref();
     m_pMouseInput = NULL;
   }
 
   if(m_p1DMouseInput) {
-    m_p1DMouseInput->Unref();
     m_p1DMouseInput = NULL;
   }
 
@@ -255,12 +252,15 @@ GArray *CDasherControl::GetAllowedValues(int iParameter) {
 
 void CDasherControl::RealizeCanvas(GtkWidget *pWidget) {
   // TODO: Pointless function - call directly from C callback.
+#ifdef DEBUG
+  std::cout << "RealizeCanvas()" << std::endl;
+#endif
   OnUIRealised();
 }
 
 void CDasherControl::StartTimer() {
-  // Start the timer loops as everything is set up
-  // Aim for 40 frames per second, computers are getting faster
+  // Start the timer loops as everything is set up.
+  // Aim for 40 frames per second, computers are getting faster.
 
   if(g_iTimeoutID == 0) {
     g_iTimeoutID = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 25, timer_callback, this, NULL);
