@@ -1,7 +1,9 @@
 #ifndef __dashercontrol_h__
 #define __dashercontrol_h__
 
-#include "../../config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "PangoCache.h"
 #include "Canvas.h"
@@ -131,7 +133,7 @@ private:
   virtual void ScanAlphabetFiles(std::vector<std::string> &vFileList);
   virtual void ScanColourFiles(std::vector<std::string> &vFileList);
   virtual void SetupPaths();
-  virtual void CreateLocalFactories();
+  virtual void CreateModules();
   virtual void SetupUI();
   virtual void CreateSettingsStore();
   virtual int GetFileSize(const std::string &strFileName);
