@@ -18,13 +18,12 @@
 // along with Dasher; if not, write to the Free Software 
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#ifdef GNOME_LIBS 
-#include <libgnomevfs/gnome-vfs.h> 
-#endif
 
 #include "dasher_action_keyboard.h"
 #ifdef WITH_MAEMO
@@ -83,7 +82,7 @@ struct _DasherEditorPrivate {
   gboolean bFileModified; // TODO: Make this work properly, export to main for quit etc
 };
 
-#define DASHER_EDITOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o), TYPE_DASHER_EDITOR, DasherEditorPrivate))
+#define DASHER_EDITOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o), DASHER_TYPE_EDITOR, DasherEditorPrivate))
 
 /* Signals */
 enum {
@@ -108,7 +107,7 @@ static void
 dasher_editor_class_init(DasherEditorClass *pClass) {
   g_type_class_add_private(pClass, sizeof(DasherEditorPrivate));
 
-  GObjectClass *pObjectClass = (GObjectClass *) pClass;
+  GObjectClass *pObjectClass = G_OBJECT_CLASS(pClass);
   pObjectClass->finalize = dasher_editor_finalize;
 
   /* Setup signals */
@@ -203,9 +202,9 @@ dasher_editor_finalize(GObject *pObject) {
 }
 
 void
-dasher_editor_initialise(DasherEditor *pSelf, DasherAppSettings *pAppSettings, DasherMain *pDasherMain, GladeXML *pGladeXML, const gchar *szFullPath) {
+dasher_editor_initialise(DasherEditor *pSelf, DasherAppSettings *pAppSettings, DasherMain *pDasherMain, GtkBuilder *pXML, const gchar *szFullPath) {
   if(DASHER_EDITOR_GET_CLASS(pSelf)->initialise)
-    DASHER_EDITOR_GET_CLASS(pSelf)->initialise(pSelf, pAppSettings, pDasherMain, pGladeXML, szFullPath);
+    DASHER_EDITOR_GET_CLASS(pSelf)->initialise(pSelf, pAppSettings, pDasherMain, pXML, szFullPath);
 }
 
 void 
