@@ -49,7 +49,9 @@ CDasher::~CDasher(void) {
   delete m_pCanvas;
 }
 
-void CDasher::CreateLocalFactories() {
+void CDasher::CreateModules() {
+  //create default set first.
+  CDasherInterfaceBase::CreateModules();
 #ifndef _WIN32_WCE
   RegisterModule(new CSocketInput(m_pEventHandler, m_pSettingsStore));
   RegisterModule(new CBTSocketInput(m_pEventHandler, m_pSettingsStore));
