@@ -10,8 +10,8 @@ public:
   virtual LRESULT OnScroll(UINT message, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
  
   virtual int GetHeightRequest();
-  virtual void Initialise(CDasherInterfaceBase *pInterface);
-  virtual void Apply(CDasherInterfaceBase *pInterface);
+  virtual void Initialise(Dasher::CDasherInterfaceBase *pInterface);
+  virtual void Apply(Dasher::CDasherInterfaceBase *pInterface);
   virtual void CreateChild(HWND hParent);
   virtual void LayoutChild(RECT &sRect);
 
