@@ -8,8 +8,8 @@ public:
   CModuleControlString(SModuleSettings *pSetting) : CModuleControl(pSetting) {};
 
   virtual int GetHeightRequest();
-  virtual void Initialise(CDasherInterfaceBase *pInterface);
-  virtual void Apply(CDasherInterfaceBase *pInterface);
+  virtual void Initialise(Dasher::CDasherInterfaceBase *pInterface);
+  virtual void Apply(Dasher::CDasherInterfaceBase *pInterface);
   virtual void CreateChild(HWND hParent);
   virtual void LayoutChild(RECT &sRect);
 
