@@ -13,6 +13,7 @@
 #define IDS_BUTTON_SETTINGS_TITLE       106
 #define IDS_FILE_SAVE                   107
 #define IDM_EXIT                        108
+#define IDS_OPTIONS_PREFS               108
 #define IDS_FILE_OPEN                   109
 #define IDS_HELLO                       110
 #define IDS_FILE_NEW                    111
@@ -108,6 +109,7 @@
 #define IDC_BUTTON_PREFS2               1019
 #define IDC_CLIPTOALPH_UNORDERED        1020
 #define IDC_THICKLINE                   1020
+#define IDC_COPYONSTOP                  1020
 #define IDC_LIST2                       1021
 #define IDC_LIST_GROUP                  1022
 #define IDC_CLIPTOALPH_ORDERED          1023
@@ -246,6 +248,8 @@
 #define ID_OPTIONS_KEYCONTROL           32815
 #define ID_OPTIONS_WINDOWPAUSE          32816
 #define ID_HELP_DASHERTUTORIAL          32817
+#define ID_HELP_GAMEMODE                32818
+#define ID_GAMEMODE                     32819
 #define IDC_STATIC                      -1
 
 // Next default values for new objects
@@ -253,8 +257,8 @@
 #ifdef APSTUDIO_INVOKED
 #ifndef APSTUDIO_READONLY_SYMBOLS
 #define _APS_NEXT_RESOURCE_VALUE        180
-#define _APS_NEXT_COMMAND_VALUE         32818
-#define _APS_NEXT_CONTROL_VALUE         1157
+#define _APS_NEXT_COMMAND_VALUE         32820
+#define _APS_NEXT_CONTROL_VALUE         1160
 #define _APS_NEXT_SYMED_VALUE           110
 #endif
 #endif
