@@ -1,20 +1,18 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#ifdef GNOME_LIBS
 #include <glib/gi18n.h>
-#endif
 
 #include "Common/Common.h"
 
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-#include <glade/glade.h>
 #include <signal.h>
-#if GLIB_CHECK_VERSION(2,6,0)
 #include <Gtk2/DasherAppSettings.h>
-#endif
+/* Just to make sure the symbols for the editors are visible. */
+#include <Gtk2/dasher_editor_internal.h>
+#include <Gtk2/dasher_editor_external.h>
 
 #ifdef WITH_MAEMO
 #include <libosso.h>
@@ -22,7 +20,6 @@
 
 // TODO: This shouldn't need to be here
 #if (defined GNOME_SPEECH || defined GNOME_A11Y)
-//#include <gnome.h>
 #include <libbonobo.h>
 #endif
 
@@ -30,20 +27,7 @@
 #include <gpe/init.h>
 #endif
 
-#ifdef GNOME_LIBS
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
-#endif
-
-// #include <libintl.h>
-// #include <locale.h>
-// #include <stdio.h>
-// #include <stdlib.h>
-// #include <iostream>
-
 #include "dasher.h"
-//#include "DasherControl.h"
-#include "dasher_lock_dialogue.h"
 #include "dasher_main.h"
 
 #ifdef WITH_GPE
@@ -177,8 +161,6 @@ int main(int argc, char *argv[]) {
   sCommandLine.szAppStyle = NULL;
   sCommandLine.szOptions = NULL;
 
-  // TODO: It would be nice to have command line parsing in version prior to goption (eg in Solaris 10)...
-#if GLIB_CHECK_VERSION(2,6,0)
   gboolean do_option_help = false;
   static const GOptionEntry options[] = {
     //   {"timedata", 'w', 0, G_OPTION_ARG_NONE, &timedata, "Write basic timing information to stdout", NULL},
@@ -212,7 +194,6 @@ int main(int argc, char *argv[]) {
     option_help();
     return 0;
   }
-#endif 
 
 #ifdef WITH_GPE
   gpe_application_init(&argc, &argv);
@@ -225,21 +206,6 @@ int main(int argc, char *argv[]) {
   osso_context = osso_initialize("dasher", PACKAGE_VERSION, TRUE, NULL);
 #endif
 
-#ifdef GNOME_LIBS
-  GnomeProgram *program = 0;
-#if GLIB_CHECK_VERSION(2,14,0)
-  program = gnome_program_init
-    (argv[0], PACKAGE_VERSION, LIBGNOMEUI_MODULE,
-     argc, argv,
-     GNOME_PARAM_GOPTION_CONTEXT, goptcontext, GNOME_PARAM_NONE);
-#else
-  program = gnome_program_init
-    (argv[0], PACKAGE_VERSION, LIBGNOMEUI_MODULE,
-     argc, argv,
-     GNOME_PARAM_NONE);
-#endif
-#endif /* GNOME_LIBS */
-
 #if (defined GNOME_SPEECH || defined GNOME_A11Y)
   if(!bonobo_is_initialized()) {
     if(!bonobo_init(&argc, argv)) {
@@ -250,7 +216,7 @@ int main(int argc, char *argv[]) {
 #endif
 
   g_set_application_name("Dasher");
-#if (!defined WITH_MAEMO) && GTK_CHECK_VERSION(2,6,0)
+#ifndef WITH_MAEMO
   gtk_window_set_default_icon_name("dasher");
 #endif
 
@@ -266,6 +232,10 @@ int main(int argc, char *argv[]) {
 
   dasher_main_show(g_pDasherMain);
 
+  // This call is just to force the linker to export the following symbols.
+  dasher_editor_internal_get_type();
+  dasher_editor_external_get_type();
+
   // 10.
   gtk_main();
 
